@@ -17,27 +17,6 @@
 *  MA  02111-1307  USA
 */
 
-/**
- * \author Jolien D. E. Creighton
- * \file
- *
- * \brief Low-level routines for manipulating frame data streams.
- *
- * ### Synopsis ###
- *
- * \code
- * #include <stdio.h>
- * #include <lal/LALFrStream.h>
- * \endcode
- *
- * A frame stream is like a file stream except that it streams along the set
- * of frames in a set of frame files.  These routines are low-level routines
- * that allow you to extract frame data.  Many of these routines have names
- * similar to the standard C file stream manipulation routines and perform
- * similar functions.
- *
- */
-
 #include <stdio.h>
 #include <lal/LALDatatypes.h>
 #include <lal/LALCache.h>
@@ -52,26 +31,120 @@ extern "C" {
 #if 0
 }       /* so that editors will match preceding brace */
 #endif
+
+/**
+ * @defgroup LALFrStream_h Header LALFrStream.h
+ * @ingroup lalframe_general
+ *
+ * \author Jolien D. E. Creighton
+ *
+ * \brief High-level routines for manipulating frame data streams.
+ *
+ * ### Synopsis ###
+ *
+ * \code
+ * #include <stdio.h>
+ * #include <lal/LALFrStream.h>
+ * \endcode
+ *
+ * ### Description ###
+ *
+ * A frame stream is like a file stream except that it streams along the set
+ * of frames in a set of frame files.  These routines are high-level routines
+ * that allow you to extract frame data.  Many of these routines have names
+ * similar to the standard C file stream manipulation routines and perform
+ * similar functions.
+ *
+ * The routines XLALFrStreamOpen() and XLALFrStreamClose() are used to open
+ * and close a frame stream.  The stream is created by XLALFrStreamOpen(),
+ * and must be a pointer to NULL before it is opened.  It must have
+ * been created prior to calling XLALFrStreamClose(), and after this call,
+ * the stream will be a pointer to  NULL.  The routine
+ * XLALFrStreamOpen() requires the user to specify the directory name of the
+ * frame files and the head names.  If the directory is  NULL, the
+ * routine uses the current director (.).  The head names specifies
+ * which files are the wanted files in the specified directory.  Wildcards are
+ * allowed.  For example, to get LLO frames only, the head names could be set
+ * to L-*.gwf.  If the head name is  NULL, the default value
+ * *.gwf is used.  The routine XLALFrStreamCacheOpen() is like
+ * XLALFrStreamOpen() except that the list of frame files is taken from a
+ * frame file cache.  [In fact, XLALFrStreamOpen() simply uses
+ * XLALFrCacheGenerate() and XLALFrStreamCacheOpen() to create the
+ * stream.]
+ *
+ * The routine XLALFrStreamSetMode() is used to change the operating mode
+ * of a frame stream, which determines how the routines try to accomodate
+ * gaps in data and requests for times when there is no data (e.g., before
+ * the beginning of the data, after the end of the data, or in some missing
+ * data).  The default mode, which is given the value
+ * #LAL_FR_STREAM_DEFAULT_MODE, prints warnings if a time requested
+ * corresponds to a time when there is no data (but then skips to the first
+ * avaliable data) and prints an info message when a gap in the data occurs
+ * (but then skips beyond the gap).  This default mode is equal to the
+ * combination
+ * #LAL_FR_STREAM_VERBOSE_MODE | #LAL_FR_STREAM_IGNOREGAP_MODE | #LAL_FR_STREAM_IGNORETIME_MODE
+ * where  #LAL_FR_STREAM_VERBOSE_MODE is equal to the combination
+ * #LAL_FR_STREAM_TIMEWARN_MODE | #LAL_FR_STREAM_GAPINFO_MODE.  Use
+ * #LAL_FR_STREAM_VERBOSE_MODE to print out warnings when requesting times
+ * with no data and print out an info message when a gap in the data is
+ * encountered.  Unless the mode is supplemented with
+ * #LAL_FR_STREAM_IGNOREGAP_MODE, gaps encountered in the data will cause
+ * a routine to exit with a non-zero status code; similarly,
+ * #LAL_FR_STREAM_IGNORETIME_MODE prevents routines from failing if a time
+ * when there is not data is requested.  Set the mode to
+ * #LAL_FR_STREAM_SILENT_MODE to suppress the warning and info messages but
+ * still cause routines to fail when data is not available.
+ * Note: the default value  #LAL_FR_STREAM_DEFAULT_MODE is assumed initially,
+ * but this is not necessarily the recommended mode --- it is adopted for
+ * compatibility reasons.
+ *
+ * The routine XLALFrStreamEnd() determines if the end-of-frame-data flag for
+ * the data stream has been set.
+ *
+ * The routine XLALFrStreamNext() advances the frame stream to the
+ * beginning of the next frame.
+ *
+ * The routine XLALFrStreamRewind() rewinds the frame stream to the first
+ * frame.
+ *
+ * The routine XLALFrStreamSeek() sets the frame stream to a specified time,
+ * or the earliest time after the specified time if that time is not available
+ * (e.g., if it is before the beginning of the frame stream or if it is in a
+ * gap in the frame data).  The routine XLALFrStreamTell() returns the
+ * current time within the frame stream.
+ *
+ * The routine XLALFrStreamGetpos() returns a structure containing the
+ * current frame stream position.  The frame stream can later be restored to
+ * this position using XLALFrStreamSetpos().
+ */
+/** @{ */
+
+/** Enum listing different stream states */
 typedef enum {
-    LAL_FR_STREAM_OK = 0,       /* nominal */
-    LAL_FR_STREAM_ERR = 1,      /* error in frame stream */
-    LAL_FR_STREAM_END = 2,      /* end of frame stream */
-    LAL_FR_STREAM_GAP = 4,      /* gap in frame stream */
-    LAL_FR_STREAM_URL = 8,      /* error opening frame URL */
-    LAL_FR_STREAM_TOC = 16      /* error reading frame TOC */
+    LAL_FR_STREAM_OK = 0,       /**< nominal */
+    LAL_FR_STREAM_ERR = 1,      /**< error in frame stream */
+    LAL_FR_STREAM_END = 2,      /**< end of frame stream */
+    LAL_FR_STREAM_GAP = 4,      /**< gap in frame stream */
+    LAL_FR_STREAM_URL = 8,      /**< error opening frame URL */
+    LAL_FR_STREAM_TOC = 16      /**< error reading frame TOC */
 } LALFrStreamState;
 
+/** Enum listing different stream modes */
 typedef enum {
-    LAL_FR_STREAM_SILENT_MODE = 0,
-    LAL_FR_STREAM_TIMEWARN_MODE = 1,    /* display warning for invalid time requests */
-    LAL_FR_STREAM_GAPINFO_MODE = 2,     /* display info for gaps in data */
-    LAL_FR_STREAM_VERBOSE_MODE = 3,     /* display warnings and info */
-    LAL_FR_STREAM_IGNOREGAP_MODE = 4,   /* ignore gaps in data */
-    LAL_FR_STREAM_IGNORETIME_MODE = 8,  /* ignore invalid times requested */
-    LAL_FR_STREAM_DEFAULT_MODE = 15,    /* ignore time/gaps but report warnings & info */
-    LAL_FR_STREAM_CHECKSUM_MODE = 16    /* ensure that file checksums are OK */
+    LAL_FR_STREAM_SILENT_MODE = 0,	/**< silent mode */
+    LAL_FR_STREAM_TIMEWARN_MODE = 1,    /**< display warning for invalid time requests */
+    LAL_FR_STREAM_GAPINFO_MODE = 2,     /**< display info for gaps in data */
+    LAL_FR_STREAM_VERBOSE_MODE = 3,     /**< display warnings and info */
+    LAL_FR_STREAM_IGNOREGAP_MODE = 4,   /**< ignore gaps in data */
+    LAL_FR_STREAM_IGNORETIME_MODE = 8,  /**< ignore invalid times requested */
+    LAL_FR_STREAM_DEFAULT_MODE = 15,    /**< ignore time/gaps but report warnings & info */
+    LAL_FR_STREAM_CHECKSUM_MODE = 16    /**< ensure that file checksums are OK */
 } LALFrStreamMode;
 
+/**
+ * This structure details the state of the frame stream.  The contents are
+ * private; you should not tamper with them!
+ */
 typedef struct tagLALFrStream {
     LALFrStreamState state;
     INT4 mode;
@@ -81,26 +154,17 @@ typedef struct tagLALFrStream {
     LALFrFile *file;
     INT4 pos;
 } LALFrStream;
-/**
- * This structure details the state of the frame stream.  The contents are
- * private; you should not tamper with them!
- */
 
-typedef struct tagLALFrStreamPos {
-    LIGOTimeGPS epoch;
-    UINT4 fnum;
-    INT4 pos;
-} LALFrStreamPos;
 /**
  * This structure contains a record of the state of a frame stream; this
  * record can be used to restore the stream to the state when the record
- * was made (provided the stream has not been closed).  The fields are:
- * <dl>
- * <dt>epoch</dt><dd> the GPS time of the open frame when the record  was made.</dd>
- * <dt>fnum</dt><dd> the file number of a list of frame files that was open when the record was made.</dd>
- * <dt>pos</dt><dd> the position within the frame file that was open when the record was made.</dd>
- * </dl>
+ * was made (provided the stream has not been closed).
  */
+typedef struct tagLALFrStreamPos {
+  LIGOTimeGPS epoch;	/**< the GPS time of the open frame when the record  was made */
+  UINT4 fnum;		/**< the file number of a list of frame files that was open when the record was made */
+  INT4 pos;		/**< the position within the frame file that was open when the record was made */
+} LALFrStreamPos;
 
 LALFrStream *XLALFrStreamCacheOpen(LALCache * cache);
 LALFrStream *XLALFrStreamOpen(const char *dirname, const char *pattern);
@@ -227,10 +291,10 @@ COMPLEX16FrequencySeries
     *XLALFrStreamInputCOMPLEX16FrequencySeries(LALFrStream * stream,
     const char *chname, const LIGOTimeGPS * epoch);
 
-/* LEGACY CODE */
+/* ---------- LEGACY CODE ---------- */
 
 /**\name Legacy Error Codes */
-/*@{*/
+/**@{*/
 #define FRAMESTREAMH_ENULL 00001
 #define FRAMESTREAMH_ENNUL 00002
 #define FRAMESTREAMH_EALOC 00004
@@ -265,61 +329,52 @@ typedef enum {
     LAL_ADC_CHAN, LAL_SIM_CHAN, LAL_PROC_CHAN
 } FrChanType;
 
-/* for backwards compatability... */
+/** @name for backwards compatability
+ *
+ * These are the various types of channel that can be specified for read/write.
+ * They are "post-processed data" (ProcDataChannel), "ADC data"
+ * (ADCDataChannel), and "simulated data" (SimDataChannel).
+ */
+/**@{*/
 #define ChannelType FrChanType
 #define ProcDataChannel LAL_PROC_CHAN
 #define ADCDataChannel  LAL_ADC_CHAN
 #define SimDataChannel  LAL_SIM_CHAN
-/**
- * These are the various types of channel that can be specified for read/write.
- * They are "post-processed data" (\c ProcDataChannel), "ADC data"
- * (\c ADCDataChannel), and "simulated data" (\c SimDataChannel).
- */
+/**@}*/
 
 #ifdef SWIG /* SWIG interface directives */
 SWIGLAL(IMMUTABLE_MEMBERS(tagFrChanIn, name));
 #endif /* SWIG */
-typedef struct tagFrChanIn {
-    const CHAR *name;
-    ChannelType type;
-} FrChanIn;
+
 /**
- * This structure specifies the channel to read as input.  The fields are:
- * <dl>
- * <dt>name</dt><dd> the name of the channel.
- * </dd><dt>type</dt><dd> the channel type.
- * </dd></dl>
+ * This structure specifies the channel to read as input.
  */
+typedef struct tagFrChanIn {
+  const CHAR *name;	/**< the name of the channel */
+  ChannelType type;	/**< the channel type */
+} FrChanIn;
 
 #ifdef SWIG /* SWIG interface directives */
 SWIGLAL(IMMUTABLE_MEMBERS(tagFrOutPar, source, description));
 #endif /* SWIG */
-typedef struct tagFrOutPar {
-    const CHAR *source;
-    const CHAR *description;
-    ChannelType type;
-    UINT4 nframes;
-    UINT4 frame;
-    UINT4 run;
-} FrOutPar;
 /**
  *
  * This structure specifies the parameters for output of data to a frame.
- * The fields are:
- * <dl>
- * <dt>source</dt><dd> the source identifier to attach to the output frame file name.</dd>
- * <dt>description</dt><dd> the description identifier to attach to the output frame file name.</dd>
- * <dt>type</dt><dd> the type of channel to create in the output frames.</dd>
- * <dt>nframes</dt><dd> the number of frames to output in the frame file.</dd>
- * <dt>frame</dt><dd> the number the first frame of output.</dd>
- * <dt>run</dt><dd> the number this data run.</dd>
- * </dl>
  * The output frame file name will be
- * \f$\langle\mbox{source}\rangle\f$<tt>-</tt>\f$\langle\mbox{description}\rangle\f$%
- * <tt>-</tt>\f$\langle\mbox{GPS start time}\rangle\f$<tt>-</tt>%
- * \f$\langle\mbox{duration}\rangle\f$<tt>.gwf</tt>.
+ * \<source\>-\<description\>-\<GPS start time\>-\<duration\>.gwf
  */
+typedef struct tagFrOutPar {
+  const CHAR *source;		/**< the source identifier to attach to the output frame file name */
+  const CHAR *description;	/**< the description identifier to attach to the output frame file name */
+  ChannelType type;		/**< the type of channel to create in the output frames */
+  UINT4 nframes;		/**< the number of frames to output in the frame file */
+  UINT4 frame;			/**< the number the first frame of output*/
+  UINT4 run;			/**< the number this data run */
+} FrOutPar;
 
+/** @name Legacy API
+ * @{
+ */
 void
 LALFrCacheOpen(LALStatus * status, LALFrStream ** output, LALCache * cache);
 
@@ -439,6 +494,9 @@ void LALFrWriteCOMPLEX8FrequencySeries(LALStatus * status,
 
 void LALFrWriteCOMPLEX16FrequencySeries(LALStatus * status,
     COMPLEX16FrequencySeries * series, FrOutPar * params, INT4 subtype);
+/**@}*/
+
+/** @} */
 
 #if 0
 {       /* so that editors will match succeeding brace */
